import uiautomation as auto
import pygetwindow as gw
import psutil
import time

def get_focused_browser_url():
    # Define process names and class names for popular browsers
    browser_info = {
        'chrome': {'process_name': 'chrome.exe', 'class_name': 'Chrome_WidgetWin_1'},
        'brave': {'process_name': 'brave.exe', 'class_name': 'Chrome_WidgetWin_1'},
        'edge': {'process_name': 'msedge.exe', 'class_name': 'Chrome_WidgetWin_1'},
        'firefox': {'process_name': 'firefox.exe', 'class_name': 'MozillaWindowClass'}
    }

    # Get the currently focused (active) window
    active_window = gw.getActiveWindow()
    if not active_window:
        return "No active window found."

    active_window_title = active_window.title.lower()
    active_window_handle = active_window._hWnd

    # Retrieve the process ID (PID) of the active window
    try:
        # Iterate over all processes to find the one with the matching window handle
        for proc in psutil.process_iter(['pid', 'name']):
            try:
                # Check if this process has a window with the active window handle
                if proc.pid == psutil.Process(proc.pid).pid:
                    # Verify if the process name matches any of the known browser process names
                    for browser, info in browser_info.items():
                        if proc.name().lower() == info['process_name']:
                            # Access the address bar in the currently focused window
                            try:
                                # Do not activate another window; rely on the existing active window
                                time.sleep(1)  # Give the active window time to stabilize

                                # Access the address bar in the currently focused window
                                browser_window = auto.ControlFromHandle(active_window_handle)
                                address_bar = browser_window.EditControl(foundIndex=1)

                                if address_bar.Exists(0, 0):
                                    url = address_bar.GetValuePattern().Value
                                    if url.startswith("http"):  # Confirm it’s a valid URL
                                        return url
                                    else:
                                        return "Detected invalid URL format in the address bar."
                                else:
                                    return f"Address bar not found in the {browser.capitalize()} window."
                            except Exception as e:
                                return f"Error retrieving URL for {browser.capitalize()}: {e}"
            except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
                pass
    except Exception as e:
        return f"Error retrieving process information: {e}"

    return "No active browser URL found."

if __name__ == "__main__":
    url = get_focused_browser_url()
    print(url if url else "Unable to retrieve the URL.")
